
#ifndef _H_BIG_INT
#define _H_BIG_INT

/** A 128-bit integer, with overloaded comparison operators. */
struct BigInt {
	unsigned long int high;
	unsigned long int low;
	BigInt(): high( 0 ), low( 0 ) {}
	BigInt( unsigned long int a, unsigned long int b ): high( a ), low( b ) {}
	void operator=( const BigInt &other ) {
		high = other.high;
		low  = other.low;
	}
};

/** Compare function used for quicksort on an array of bigints */
int bigint_compare( const void *a, const void *b ) {
	const BigInt left  = *(BigInt*)a;
	const BigInt right = *(BigInt*)b;
	if( left.high == right.high ) {
		if( left.low == right.low )
			return 0;
		else
			return (left.low > right.low) ? 1 : -1;
	} else
		return (left.high > right.high) ? 1 : -1;
}

bool operator>( const BigInt &left, const BigInt &right ) {
	return bigint_compare( &left, &right ) > 0;
}

bool operator<( const BigInt &left, const BigInt &right ) {
	return bigint_compare( &left, &right ) < 0;
}

bool operator==( const BigInt &left, const BigInt &right ) {
	return left.high == right.high && left.low == right.low;
}

bool operator!=( const BigInt &left, const BigInt &right ) {
	return left.high != right.high || left.low != right.low;
}

bool operator>=( const BigInt &left, const BigInt &right ) {
	return left == right || ( left > right );
}

bool operator<=( const BigInt &left, const BigInt &right ) {
	return left == right || ( left < right );
}

#endif

