#include "MachineInfo.hpp"

MachineInfo::MachineInfo() : P( ULONG_MAX ) {}

const MachineInfo& MachineInfo::getInstance() {
	//first fast check
	if( initialised ) return instance;
	//instance might not have been created, get lock
	pthread_mutex_lock( &mutex );
	//check again
	if( !initialised ) {
		//init instance
		load();
	}
	//return lock
	pthread_mutex_unlock( &mutex );
	//and return instance
	return instance;
}

void MachineInfo::load() {
	//create info stream
	std::ifstream info;
	//open info file
	info.open( "hardware.info" );
	if( !info.is_open() ) {
		std::cerr << "Warning: \tcould not open hardware.info, setting default values for MachineInfo" << std::endl;
		std::cerr << "         \tsetting the number of available cores to 1" << std::endl;
		instance.P = 1;
	} else {
		info >> instance.P; std::cout << "MachineInfo: number of available cores set to " << instance.P << std::endl;
		std::string line;
		getline( info, line );
		getline( info, line );
		while( info.good() ) {
			std::cerr << "MachineInfo: ignoring extra line " << line << std::endl;
			std::string line;
			getline( info, line );
		}
	}
	instance.initialised = true;
}

unsigned long int MachineInfo::cores() const { return P; }

MachineInfo MachineInfo::instance;

bool MachineInfo::initialised = false;

pthread_mutex_t MachineInfo::mutex = PTHREAD_MUTEX_INITIALIZER;

