
#include <string>
#include <fstream>
#include <iostream>
#include <pthread.h>
#include <limits.h>

#ifndef _H_MACHINEINFO
#define _H_MACHINEINFO

/** Singleton class to get info on the current system */
class MachineInfo {

   private:
	/** Singleton class instance */
	static MachineInfo instance;

	/** Mutex to prevent concurrent accesses whenever appropriate */
	static pthread_mutex_t mutex;

	/** Whether the instance is initialised */
	static bool initialised;

	/** Base constructor (will load invalid values) */
	MachineInfo();

   protected:
	/** The number of available cores on this machine */
	unsigned long int P;

	/** Intialises the singleton instance */
	static void load();

   public:
	/** Gets a singleton instance */
	static const MachineInfo& getInstance();
	
	/** The number of available cores */
	unsigned long int cores() const;
	
};

#endif

